@@ -6,10 +6,12 @@
 /**
  * since std::mutex is *VERY SLOW* on OSX, we use
  * this there instead.
- * Taken from (http://stackoverflow.com/questions/22899053/why-is-stdmutex-so-slow-on-osx)
+ * Taken from
+ * (http://stackoverflow.com/questions/22899053/why-is-stdmutex-so-slow-on-osx)
  */
 class spin_lock {
-    std::atomic<bool> _lock;
+  std::atomic<bool> _lock;
+
 public:
   spin_lock(const spin_lock&) = delete;
   spin_lock& operator=(const spin_lock&) = delete;
@@ -17,21 +19,19 @@ public:
   spin_lock() : _lock(false) {}
 
   class scoped_lock {
-    spin_lock &_lock;
+    spin_lock& _lock;
 
   public:
     scoped_lock(const scoped_lock&) = delete;
     scoped_lock& operator=(const scoped_lock&) = delete;
 
-    scoped_lock(spin_lock &lock) : _lock(lock) {
+    scoped_lock(spin_lock& lock) : _lock(lock) {
       bool expect = false;
       while (!_lock._lock.compare_exchange_weak(expect, true)) {
         expect = false;
       }
     }
-    ~scoped_lock() {
-      _lock._lock = false;
-    }
+    ~scoped_lock() { _lock._lock = false; }
   };
 };
 
