@@ -2,8 +2,8 @@
 #define STADEN_UTILS
 
 extern "C" {
-#include "io_lib/scram.h"
 #include "io_lib/os.h"
+#include "io_lib/scram.h"
 #undef max
 #undef min
 }
@@ -11,10 +11,10 @@ extern "C" {
 #include <cstdlib>
 
 namespace staden {
-    namespace utils {
-        bam_seq_t* bam_init();
-        void bam_destroy(bam_seq_t* b);
-    }
-}
+namespace utils {
+bam_seq_t* bam_init();
+void bam_destroy(bam_seq_t* b);
+} // namespace utils
+} // namespace staden
 
 #endif // STADEN_UTILS
