@@ -13,8 +13,8 @@ p2="$tmpdir/p2.fa"
 mkfifo $p1
 mkfifo $p2
 
-i1=`ls $base*I1*`
-wrapper <(cat $base*I1*) <(cat $base*RA*) >> $p1 2>> $p2 &
+i1=`ls $base/*I1*`
+wrapper <(cat $base/*I1*) <(cat $base/*RA*) >> $p1 2>> $p2 &
 
 echo "Running command [${new_cmd} -1 $p1 -2 $p2 -r $i1]"
 ${new_cmd} -1 $p1 -2 $p2 -r $i1
