@@ -7,10 +7,9 @@ uint32_t getGeneId(spp::sparse_hash_map<uint32_t, uint32_t> &txpToGeneMap,
     return txpToGeneMap.at(tId);
   }
   else{
-    std::cerr << "Out of Range error for txp to gene Map: "
-              << '\n' << std::flush;
-    std::cerr << tId << "\t not found" << std::flush;
-    exit(1);
+    std::cerr << "\n\n\nOut of Range error for txp to gene Map: "
+              << tId << "\t not found" << std::flush;
+    exit(74);
   }
 }
 
@@ -18,7 +17,10 @@ uint32_t getGeneId(spp::sparse_hash_map<uint32_t, uint32_t> &txpToGeneMap,
 // choosing list for edges and vector for adjacency container
 void graphFromCell(std::vector<TGroupT>& txpGroups,
                    std::vector<UGroupT>& umiGroups,
-                   alevin::graph::Graph& g) {
+                   alevin::graph::Graph& g,
+                   uint32_t umiEditDistance, uint32_t umiLength,
+                   std::atomic<uint64_t>& totalUniEdgesCounts,
+                   std::atomic<uint64_t>& totalBiEdgesCounts) {
   using namespace alevin::graph;
   spp::sparse_hash_map<uint32_t, std::vector<uint32_t>> tidMap;
 
@@ -34,6 +36,7 @@ void graphFromCell(std::vector<TGroupT>& txpGroups,
   // alevin kmer object
   alevin::types::AlevinUMIKmer umiObj;
   spp::sparse_hash_map<VertexT, uint32_t, boost::hash<VertexT>> vertexIndexMap;
+  vertexIndexMap.reserve(numClasses);
 
   //iterating over all eqclasses
   for (size_t eqId=0; eqId<numClasses; eqId++) {
@@ -51,21 +54,26 @@ void graphFromCell(std::vector<TGroupT>& txpGroups,
       uint32_t v1 = alevin::graph::getVertexIndex(vertexIndexMap, node);
 
       for ( size_t uId_second=uId+1; uId_second<numUmis; uId_second++ ){
+
+        //check if two UMI can be connected
+        EdgeType edge = alevin::graph::hasEdge( umiSeqCounts[uId], umiSeqCounts[uId_second], umiEditDistance, umiLength );
+        if ( edge == EdgeType::NoEdge ) { continue; }
+
         VertexT node_second (static_cast<uint32_t>(eqId), static_cast<uint32_t>(uId_second));
         uint32_t v2 = alevin::graph::getVertexIndex(vertexIndexMap, node_second);
 
-        //check if two UMI can be connected
-        EdgeType edge = alevin::graph::hasEdge( umiSeqCounts[uId], umiSeqCounts[uId_second]);
-
         switch ( edge ) {
         case EdgeType::BiDirected:
+          totalBiEdgesCounts += 1;
           g.add_edge(v1, v2);
           g.add_edge(v2, v1);
           break;
         case EdgeType::XToY:
+          totalUniEdgesCounts += 1;
           g.add_edge(v1, v2);
           break;
         case EdgeType::YToX:
+          totalUniEdgesCounts += 1;
           g.add_edge(v2, v1);
           break;
         case EdgeType::NoEdge:
@@ -104,24 +112,27 @@ void graphFromCell(std::vector<TGroupT>& txpGroups,
           uint32_t v1 = alevin::graph::getVertexIndex(vertexIndexMap, node);
 
           for ( size_t uId_second=0; uId_second<num2Umis; uId_second++ ){
+            //check if two UMI can be connected
+            EdgeType edge = alevin::graph::hasEdge( umiSeqCounts[uId], umi2SeqCounts[uId_second], umiEditDistance, umiLength );
             VertexT node_second (static_cast<uint32_t>(eq2Id), static_cast<uint32_t>(uId_second));
-            if ( node == node_second ) {
+            if ( node == node_second or edge == EdgeType::NoEdge ) {
               continue;
             }
-            uint32_t v2 = alevin::graph::getVertexIndex(vertexIndexMap, node_second);
 
-            //check if two UMI can be connected
-            EdgeType edge = alevin::graph::hasEdge( umiSeqCounts[uId], umi2SeqCounts[uId_second]);
+            uint32_t v2 = alevin::graph::getVertexIndex(vertexIndexMap, node_second);
 
             switch ( edge ) {
             case EdgeType::BiDirected:
+              totalBiEdgesCounts += 1;
               g.add_edge(v1, v2);
               g.add_edge(v2, v1);
               break;
             case EdgeType::XToY:
+              totalUniEdgesCounts += 1;
               g.add_edge(v1, v2);
               break;
             case EdgeType::YToX:
+              totalUniEdgesCounts += 1;
               g.add_edge(v2, v1);
               break;
             case EdgeType::NoEdge:
@@ -143,8 +154,8 @@ void graphFromCell(std::vector<TGroupT>& txpGroups,
 void collapseVertices(uint32_t vertex,
                       alevin::graph::Graph& g,
                       std::vector<TGroupT>& txpGroups,
-                      uint32_t& chosenTxp,
-                      std::vector<uint32_t>& largestMcc) {
+                      std::vector<uint32_t>& largestMcc,
+                      spp::sparse_hash_set<uint32_t>& processedSet) {
   uint32_t eqclassId = g.getEqclassId(vertex);
   for (uint32_t txp: txpGroups[eqclassId]){
     std::deque<uint32_t> bfsList;
@@ -160,7 +171,7 @@ void collapseVertices(uint32_t vertex,
       currentMcc.emplace_back(cv);
 
       for (auto nextVertex: g.getNeighbors(vertex)) {
-        if (visitedSet.contains(nextVertex)) {
+        if (visitedSet.contains(nextVertex) || !processedSet.contains(nextVertex)) {
           continue;
         }
         else{
@@ -168,8 +179,8 @@ void collapseVertices(uint32_t vertex,
         }
 
         // extract transcripts from new vertex
-        eqclassId = g.getEqclassId( nextVertex );
-        for (uint32_t ntxp: txpGroups[eqclassId]) {
+        uint32_t neqclassId = g.getEqclassId( nextVertex );
+        for (uint32_t ntxp: txpGroups[neqclassId]) {
           if (ntxp == txp){
             bfsList.emplace_back(nextVertex);
             break;
@@ -180,30 +191,35 @@ void collapseVertices(uint32_t vertex,
 
     if (largestMcc.size() < currentMcc.size()) {
       largestMcc = currentMcc;
-      chosenTxp = txp;
     }
   } //end-for
 }
 
-void getNumMolecules(alevin::graph::Graph& g,
-                     std::vector<TGroupT>& txpGroups,
-                     spp::sparse_hash_map<uint32_t, uint32_t>& t2gMap,
-                     std::vector<SalmonEqClass>& salmonEqclasses){
+void getNumMoleculesWithArborescence(alevin::graph::Graph& g,
+                                     std::vector<TGroupT>& txpGroups,
+                                     std::vector<UGroupT>& umiGroups,
+                                     spp::sparse_hash_map<uint32_t, uint32_t>& t2gMap,
+                                     std::vector<spp::sparse_hash_map<uint16_t, uint16_t>>& arboEqClassCount,
+                                     std::vector<SalmonEqClass>& salmonEqclasses){
   // get connected components
   std::vector<uint32_t> component;
   uint32_t numComps = g.connected_components(component);
   spp::sparse_hash_map<std::vector<uint32_t>,
-                       uint32_t,
+                       std::pair<uint32_t, spp::sparse_hash_map<uint16_t, uint16_t>>,
                        boost::hash<std::vector<uint32_t>>> eqclassHash;
 
+
   // making sets of relevant connected vertices
   std::vector<std::vector<uint32_t>> comps (numComps);
   for (size_t i=0; i<component.size(); i++) {
     comps[component[i]].emplace_back(static_cast<uint32_t>(i));
   }
 
+  spp::sparse_hash_map<size_t, uint32_t> compCounter;
   // iterating over connected components
   for (auto& comp: comps) {
+    compCounter[comp.size()] += 1;
+
     // more than one vertex in the component
     if ( comp.size() > 1 ) {
       spp::sparse_hash_set<uint32_t> vset(comp.begin(), comp.end());
@@ -211,11 +227,11 @@ void getNumMolecules(alevin::graph::Graph& g,
       while ( vset.size() != 0 ){
         std::vector<uint32_t> bestMcc;
         for (uint32_t vertex: vset) {
-          uint32_t coveringTxp;
           std::vector<uint32_t> newMcc;
 
           collapseVertices(vertex, g, txpGroups,
-                           coveringTxp, newMcc);
+                           newMcc,
+                           vset);
           //choose the longer collapse: Greedy
           if (bestMcc.size() < newMcc.size()) {
             bestMcc = newMcc;
@@ -225,8 +241,8 @@ void getNumMolecules(alevin::graph::Graph& g,
         tsl::hopscotch_map<uint32_t, uint32_t> globalTxpCounts;
         for (size_t vId=0; vId<bestMcc.size(); vId++){
           uint32_t vertex = bestMcc[vId];
-          std::unordered_set<uint32_t> localTxps;
           uint32_t eqclassId = g.getEqclassId(vertex);
+
           for (uint32_t txp: txpGroups[eqclassId]){
             globalTxpCounts[txp] += 1;
           }
@@ -243,11 +259,145 @@ void getNumMolecules(alevin::graph::Graph& g,
 
         if( globalTxps.size() == 0 ) {
           std::cerr << "can't find a representative transcript for a molecule\n"
-                    << "Please report this on github";
-          exit(1);
+                    << "Please report this on github ";
+          exit(74);
         }
 
+        uint16_t readspmol {0};
         for (auto rv: bestMcc){
+          uint32_t eId = g.getEqclassId(rv);
+          uint32_t uId = g.getUId(rv);
+          auto it = umiGroups[eId].begin();
+          std::advance(it, uId);
+          readspmol += it->second;
+          vset.erase(rv);
+        }
+
+        spp::sparse_hash_set<uint32_t> globalGenes;
+        for(auto txp: globalTxps){
+          uint32_t geneId = getGeneId(t2gMap, txp);
+          globalGenes.insert(geneId);
+        }
+
+        std::vector<uint32_t> genesVec (globalGenes.begin(),
+                                        globalGenes.end());
+        std::sort (genesVec.begin(), genesVec.end());
+        eqclassHash[genesVec].first += 1;
+        eqclassHash[genesVec].second[readspmol] += 1;
+      }//end-while
+    } // end-if comp.size()>1
+    else{
+      assert(comp.size() == 1);
+      uint32_t vertex = comp[0];
+      uint32_t eId = g.getEqclassId(vertex);
+      uint32_t uId = g.getUId(vertex);
+      auto it = umiGroups[eId].begin();
+      std::advance(it, uId);
+      uint16_t readspmol = it->second;
+
+      uint32_t eqclassId = g.getEqclassId(vertex);
+      TGroupT txps = txpGroups[eqclassId];
+
+      spp::sparse_hash_set<uint32_t> genes;
+      for (auto txp: txps) {
+        uint32_t gId = getGeneId(t2gMap, txp);
+        genes.insert(gId);
+      }
+
+      assert(genes.size() > 0);
+
+      std::vector<uint32_t> genesVec (genes.begin(), genes.end());
+      std::sort (genesVec.begin(), genesVec.end());
+      eqclassHash[genesVec].first += 1;
+      eqclassHash[genesVec].second[readspmol] += 1;
+    }//end-else comp.size()==1
+  } //end-outer for comps iterator
+
+  for (auto& it: eqclassHash) {
+    SalmonEqClass eqclass = {
+      it.first,
+      it.second.first,
+    };
+
+    salmonEqclasses.emplace_back(eqclass);
+    arboEqClassCount.emplace_back(it.second.second);
+  }
+}
+
+void getNumMolecules(alevin::graph::Graph& g,
+                     std::vector<TGroupT>& txpGroups,
+                     std::vector<UGroupT>& umiGroups,
+                     spp::sparse_hash_map<uint32_t, uint32_t>& t2gMap,
+                     std::vector<SalmonEqClass>& salmonEqclasses){
+  // get connected components
+  std::vector<uint32_t> component;
+  uint32_t numComps = g.connected_components(component);
+  spp::sparse_hash_map<std::vector<uint32_t>,
+                       uint32_t,
+                       boost::hash<std::vector<uint32_t>>> eqclassHash;
+
+
+  // making sets of relevant connected vertices
+  std::vector<std::vector<uint32_t>> comps (numComps);
+  for (size_t i=0; i<component.size(); i++) {
+    comps[component[i]].emplace_back(static_cast<uint32_t>(i));
+  }
+
+  spp::sparse_hash_map<size_t, uint32_t> compCounter;
+  // iterating over connected components
+  for (auto& comp: comps) {
+    compCounter[comp.size()] += 1;
+
+    // more than one vertex in the component
+    if ( comp.size() > 1 ) {
+      spp::sparse_hash_set<uint32_t> vset(comp.begin(), comp.end());
+
+      while ( vset.size() != 0 ){
+        std::vector<uint32_t> bestMcc;
+        for (uint32_t vertex: vset) {
+          std::vector<uint32_t> newMcc;
+
+          collapseVertices(vertex, g, txpGroups,
+                           newMcc,
+                           vset);
+          //choose the longer collapse: Greedy
+          if (bestMcc.size() < newMcc.size()) {
+            bestMcc = newMcc;
+          }
+        }// end-vset for
+
+        tsl::hopscotch_map<uint32_t, uint32_t> globalTxpCounts;
+        for (size_t vId=0; vId<bestMcc.size(); vId++){
+          uint32_t vertex = bestMcc[vId];
+          uint32_t eqclassId = g.getEqclassId(vertex);
+
+          for (uint32_t txp: txpGroups[eqclassId]){
+            globalTxpCounts[txp] += 1;
+          }
+        }//end-mcc for
+
+        // only transcripts that occur in *every* vertex, and hence
+        // have a count of bestMcc.size(), are in the proper intersection.
+        uint32_t requiredCount = bestMcc.size();
+        std::vector<uint32_t> globalTxps;
+        globalTxps.reserve(globalTxpCounts.size());
+        for (auto kv : globalTxpCounts) {
+          if (kv.second == requiredCount) { globalTxps.push_back(kv.first); }
+        }
+
+        if( globalTxps.size() == 0 ) {
+          std::cerr << "can't find a representative transcript for a molecule\n"
+                    << "Please report this on github ";
+          exit(74);
+        }
+
+        uint16_t readspmol {0};
+        for (auto rv: bestMcc){
+          uint32_t eId = g.getEqclassId(rv);
+          uint32_t uId = g.getUId(rv);
+          auto it = umiGroups[eId].begin();
+          std::advance(it, uId);
+          readspmol += it->second;
           vset.erase(rv);
         }
 
@@ -266,6 +416,12 @@ void getNumMolecules(alevin::graph::Graph& g,
     else{
       assert(comp.size() == 1);
       uint32_t vertex = comp[0];
+      uint32_t eId = g.getEqclassId(vertex);
+      uint32_t uId = g.getUId(vertex);
+      auto it = umiGroups[eId].begin();
+      std::advance(it, uId);
+      uint16_t readspmol = it->second;
+
       uint32_t eqclassId = g.getEqclassId(vertex);
       TGroupT txps = txpGroups[eqclassId];
 
@@ -339,7 +495,7 @@ void assignTiers(std::vector<TGroupT>& txpGroups,
         if (!vertexIndices.contains(gene_from) or
             !vertexIndices.contains(gene_to)){
           std::cerr<<"ERROR: Tier creation can't match indexToGene"<<std::flush;
-          std::exit(1);
+          std::exit(74);
         }
         uint32_t gfromIndex = vertexIndices[gene_from];
         uint32_t gtoIndex = vertexIndices[gene_to];
@@ -375,7 +531,7 @@ void assignTiers(std::vector<TGroupT>& txpGroups,
 
   if(component.size() != vertexIndices.size()){
     std::cerr<<"ERROR: tiers size doesn't match";
-    std::exit(1);
+    std::exit(74);
   }
 
   // iterating over connected components and assigning tiers
@@ -387,7 +543,7 @@ void assignTiers(std::vector<TGroupT>& txpGroups,
                  <<" gene Index > indexToGene size"
                  << indexToGene.size()
                  << std::flush;
-        std::exit(1);
+        std::exit(74);
       }
       uint32_t gene = indexToGene[geneIndex];
       if (tiers[gene] == 1){
@@ -409,23 +565,41 @@ bool dedupClasses(std::vector<double>& geneAlphas,
                   std::vector<TGroupT>& txpGroups,
                   std::vector<UGroupT>& umiGroups,
                   std::vector<SalmonEqClass>& salmonEqclasses,
+                  uint32_t umiLength,
                   spp::sparse_hash_map<uint32_t, uint32_t>& txpToGeneMap,
                   std::vector<uint8_t>& tiers,
-                  GZipWriter& gzw,
-                  bool dumpUmiGraph){
+                  GZipWriter& gzw, uint32_t umiEditDistance,
+                  bool dumpUmiGraph, std::string& trueBarcodeStr,
+                  std::vector<spp::sparse_hash_map<uint16_t, uint16_t>>& arboEqClassCount,
+                  bool dumpArborescences,
+                  std::atomic<uint64_t>& totalUniEdgesCounts,
+                  std::atomic<uint64_t>& totalBiEdgesCounts){
   // make directed graph from eqclasses
   alevin::graph::Graph g;
-  graphFromCell(txpGroups, umiGroups, g);
+  graphFromCell(txpGroups, umiGroups, g,
+                umiEditDistance,
+                umiLength,
+                totalUniEdgesCounts,
+                totalBiEdgesCounts);
 
   if (dumpUmiGraph){
-    gzw.writeUmiGraph(g);
+    gzw.writeUmiGraph(g, trueBarcodeStr);
   }
 
   // assign tiers to the genes
   assignTiers(txpGroups, txpToGeneMap, tiers);
 
-  // make gene based eqclasses
-  getNumMolecules(g, txpGroups, txpToGeneMap, salmonEqclasses);
+  if (dumpArborescences) {
+    // make gene based eqclasses
+    getNumMoleculesWithArborescence(g, txpGroups, umiGroups,
+                                    txpToGeneMap, arboEqClassCount,
+                                    salmonEqclasses);
+  } else {
+    // make gene based eqclasses
+    getNumMolecules(g, txpGroups, umiGroups,
+                    txpToGeneMap,
+                    salmonEqclasses);
+  }
 
   for( auto& eqclass: salmonEqclasses ) {
     if ( eqclass.labels.size() == 1 ) {
@@ -434,9 +608,14 @@ bool dedupClasses(std::vector<double>& geneAlphas,
     }
     else if (eqclass.labels.size() == 0){
       std::cerr<<"Eqclasses with No gene labels\n";
-      exit(1);
+      return false;
     }
   }
 
+  if (dumpArborescences and arboEqClassCount.size() != salmonEqclasses.size()) {
+    std::cerr<<"Incorrect Arborescence features\n";
+    return false;
+  }
+
   return true;
 }
